import {createAsyncThunk, createSlice} from '@reduxjs/toolkit';
import type {PayloadAction} from '@reduxjs/toolkit';
import {RootState} from '../../store/store';
import IAlbum from '../../interfaces/IAlbum';
import {albumApi} from '../../api/albumApi';
import axios from 'axios';

interface AlbumState {
	albums: IAlbum[];
	loading: boolean;
}

const initialState: AlbumState = {
	albums: [],
	loading: false,
};

export const getAlbums = createAsyncThunk('getAlbums', async () => {
	return await albumApi.get();
});

export const getAlbumsByQueryParams = createAsyncThunk(
	'getAlbumsByArtist',
	async (artisId: string) => {
		const res = await axios.get(`${import.meta.env.VITE_MY_URL}/albums`, {
			params: {
				artist: artisId,
			},
		});
		return res.data;
	}
);

export const addAlbum = createAsyncThunk(
	'addAlbum',
	async (newAlbum: FormData) => {
		const response = await axios({
			method: 'post',
			url: `${import.meta.env.VITE_MY_URL}/albums`,
			data: newAlbum,
			headers: {
				'Content-Type': `multipart/form-data; `,
			},
		});
		return response.data;
	}
);

export const publishAlbum = createAsyncThunk(
	'publishAlbum',
	async (id: string) => {
		try {
			const response = await axios.post(
				`${import.meta.env.VITE_MY_URL}/albums/${id}/publish`
			);
			return response.data;
		} catch (err) {
			console.log(err);
		}
	}
);

export const deleteAlbum = createAsyncThunk(
	'deletAlbum',
	async (data: {id: string; token: string}) => {
		const response = await axios.delete(
			`${import.meta.env.VITE_MY_URL}/albums/${data.id}`,
			{
				headers: {
					Authorization: data.token,
				},
			}
		);
		return response.data;
	}
);

export const albumSlice = createSlice({
	name: 'artist',
	initialState,
	reducers: {},
	extraReducers: (builder) => {
		builder
			.addCase(getAlbumsByQueryParams.pending, (state, action) => {
				state.loading = true;
			})
			.addCase(getAlbumsByQueryParams.rejected, (state, action) => {
				state.loading = false;
			})
			.addCase(
				getAlbumsByQueryParams.fulfilled,
				(state, {payload}: PayloadAction<IAlbum[]>) => {
					state.albums = payload;
				}
			)
			.addCase(addAlbum.pending, (state, action) => {
				state.loading = true;
			})
			.addCase(addAlbum.rejected, (state, action) => {
				state.loading = false;
			})
			.addCase(addAlbum.fulfilled, (state, action) => {
				state.loading = false;
			})
			.addCase(publishAlbum.pending, (state, action) => {
				state.loading = true;
			})
			.addCase(publishAlbum.rejected, (state, action) => {
				state.loading = false;
			})
			.addCase(publishAlbum.fulfilled, (state, action) => {
				state.loading = false;
			})
			.addCase(getAlbums.pending, (state, action) => {
				state.loading = true;
			})
			.addCase(getAlbums.rejected, (state, action) => {
				state.loading = false;
			})
			.addCase(getAlbums.fulfilled, (state, action) => {
				state.loading = false;
				state.albums = action.payload;
			});
	},
});

export const {} = albumSlice.actions;

export const selectAlbums = (state: RootState) => state.artist.artists;

export default albumSlice.reducer;
